import sys

try:
    from django.conf import settings
    from django.test.utils import get_runner

    settings.configure(
        DEBUG=True,
        USE_TZ=True,
        DATABASES={
            "default": {
                "ENGINE": "django.db.backends.sqlite3",
            }
        },
        ROOT_URLCONF="bgfiles.urls",
        INSTALLED_APPS=[
            "django.contrib.auth",
            "django.contrib.contenttypes",
            "django.contrib.sites",
            "bgfiles",
        ],
        SITE_ID=1,
        MIDDLEWARE_CLASSES=(),
    )

    try:
        import django
        setup = django.setup
    except AttributeError:
        pass
    else:
        setup()

except ImportError:
    import traceback
    traceback.print_exc()
    raise ImportError("To fix this error, run: pip install -r requirements-test.txt")


def run_tests(*test_args):
    if not test_args:
        test_args = ['tests']

    # Run tests
    TestRunner = get_runner(settings)
    test_runner = TestRunner()

    failures = test_runner.run_tests(test_args)

    if failures:
        sys.exit(bool(failures))


if __name__ == '__main__':
    run_tests(*sys.argv[1:])



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function, division, absolute_import
from bgfiles import toolbox
from bgfiles.views import serve_file, SuspiciousToken, SignatureHasExpired, UserIsNotRequester
from django.contrib.auth.models import User
from django.core.exceptions import PermissionDenied, SuspiciousOperation
from django.core.files.base import ContentFile
from django.core.signing import SignatureExpired
from django.http import Http404
from django.test import SimpleTestCase, TestCase, RequestFactory, override_settings
import mock
from .utils import WithTempDir


class VerifyExceptionTypeTest(SimpleTestCase):

    def test(self):
        self.assertTrue(issubclass(SuspiciousToken, SuspiciousOperation))
        self.assertTrue(issubclass(SignatureHasExpired, PermissionDenied))
        self.assertTrue(issubclass(UserIsNotRequester, PermissionDenied))


class ServeFileTest(WithTempDir, TestCase):

    def setUp(self):
        self.factory = RequestFactory()
        self.user = User.objects.create_user('abc', 'mail@example.com', 'abc')

    def test_request_does_not_exist(self):
        request = toolbox.add_request('abc', requester=self.user)
        token = toolbox.create_token(request)
        request.delete()
        http_request = self.factory.get('/something')
        http_request.user = self.user
        self.assertRaises(Http404, serve_file, http_request, token)

    @mock.patch('bgfiles.views.toolbox.decode')
    def test_token_expired(self, decode_mock):
        decode_mock.side_effect = SignatureExpired
        with override_settings(MEDIA_ROOT=self.tempdir, MEDIA_URL='/'):
            request = toolbox.add_request('abc', requester=self.user)
            toolbox.attach_file(request, ContentFile(b'abc'))
            token = toolbox.create_token(request)
            http_request = self.factory.get('/something')
            http_request.user = self.user
            self.assertRaises(SignatureHasExpired, serve_file, http_request, token)

    def test_invalid_token(self):
        with override_settings(MEDIA_ROOT=self.tempdir, MEDIA_URL='/'):
            request = toolbox.add_request('abc', requester=self.user)
            toolbox.attach_file(request, ContentFile(b'abc'))
            token = toolbox.create_token(request)
            http_request = self.factory.get('/something')
            http_request.user = self.user
            self.assertRaises(SuspiciousToken, serve_file, http_request, token + 'a')

    def test_other_user(self):
        other_user = User.objects.create_user('oho', 'mymail@example.com', 'nono')
        with override_settings(MEDIA_ROOT=self.tempdir, MEDIA_URL='/'):
            request = toolbox.add_request('abc', requester=self.user)
            toolbox.attach_file(request, ContentFile(b'abc'))
            token = toolbox.create_token(request)
            http_request = self.factory.get('/something')
            http_request.user = other_user
            self.assertRaises(UserIsNotRequester, serve_file, http_request, token)

    def test_other_user_with_no_verification(self):
        other_user = User.objects.create_user('oho', 'mymail@example.com', 'nono')
        with override_settings(MEDIA_ROOT=self.tempdir, MEDIA_URL='/'):
            request = toolbox.add_request('abc', requester=self.user)
            toolbox.attach_file(request, ContentFile(b'abc'))
            token = toolbox.create_token(request)
            http_request = self.factory.get('/something')
            http_request.user = other_user
            response = serve_file(http_request, token, verify_requester=False)
            self.assertEqual(response.status_code, 200)

    def test_other_user_with_general_token(self):
        other_user = User.objects.create_user('oho', 'mymail@example.com', 'nono')
        with override_settings(MEDIA_ROOT=self.tempdir, MEDIA_URL='/'):
            request = toolbox.add_request('abc', requester=self.user)
            toolbox.attach_file(request, ContentFile(b'abc'))
            token = toolbox.create_general_token(request)
            http_request = self.factory.get('/something')
            http_request.user = other_user
            # With no verification and no requester required
            response = serve_file(http_request, token, require_requester=False, verify_requester=False)
            self.assertEqual(response.status_code, 200)
            # Now try with simply not requiring the requester
            http_request.user = other_user
            self.assertRaises(UserIsNotRequester, serve_file, http_request, token, require_requester=False)
            # Now try with requiring the requester but not verifying
            http_request.user = other_user
            self.assertRaises(SuspiciousToken, serve_file, http_request, token, verify_requester=False)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function, division, absolute_import
from shutil import rmtree
from tempfile import mkdtemp


class WithTempDir(object):
    tempdir = None

    @classmethod
    def setUpClass(cls):
        super(WithTempDir, cls).setUpClass()
        cls.tempdir = mkdtemp()

    @classmethod
    def tearDownClass(cls):
        if cls.tempdir:
            rmtree(cls.tempdir)
        super(WithTempDir, cls).tearDownClass()



# -*- coding: utf-8 -*-
"""
Tools for dealing with bgfiles functionality.

You could use the bgfiles models directly and build your own tooling around it, but this toolbox
should provide everything you need, safe for improved serving of files to the end user.

"""
from __future__ import unicode_literals, print_function, division, absolute_import
from datetime import timedelta
from django.conf import settings
from django.core.files import File
from django.core.serializers.json import DjangoJSONEncoder
from django.core.signing import dumps, loads, BadSignature
from django.http import HttpResponse, Http404
from django.utils import timezone
import json
from .http import create_content_disposition
from .models import FileRequest


class JSONSerializer(object):
    """Custom serializer for tokens supporting UUIDs and the like."""

    def dumps(self, obj):
        return json.dumps(obj, separators=(',', ':'), cls=DjangoJSONEncoder).encode('latin-1')

    def loads(self, data):
        return json.loads(data.decode('latin-1'))


class Signer(object):
    """Default signer class, wraps ``django.core.signing`` functionality and settings.

    If you supply your own signer, you don't need to subclass this class, but you should provide compatible
    ``load`` and ``dump`` methods and a ``max_age`` property.

    """

    DEFAULT_SALT = 'bgfiles'
    DEFAULT_SERIALIZER = JSONSerializer

    def __init__(self, key=None, salt=DEFAULT_SALT, serializer=DEFAULT_SERIALIZER, compress=False, max_age=None):
        self.key = key
        self.salt = salt
        self.serializer = serializer
        self.compress = compress
        self.max_age = max_age

    def load(self, token):
        """Load the data from the token.

        This simply wraps ``django.core.signing.loads``. We don't wrap the exceptions raised by this function.

        :param token: token
        :type token: str
        :return: the data contained in the token
        """
        return loads(token, key=self.key, salt=self.salt, serializer=self.serializer, max_age=self.max_age)

    def dump(self, data):
        """Create a token with the data.

        This simply wraps ``django.core.signing.loads``. We don't wrap the exceptions raised by this function.

        :param data: data
        :type data: dict
        :return: the token
        """
        return dumps(data, key=self.key, salt=self.salt, serializer=self.serializer, compress=self.compress)


def get_default_signer():
    """Retrieve the configured default signer.

    Either we use the configured signer instance or we create our own instance of our default Signer. The parameters
    for that instance can be configured through your Django project settings.

    """
    signer = getattr(settings, 'BGFILES_DEFAULT_SIGNER', None)
    if not signer:
        key = getattr(settings, 'BGFILES_DEFAULT_KEY', settings.SECRET_KEY)
        salt = getattr(settings, 'BGFILES_DEFAULT_SALT', Signer.DEFAULT_SALT)
        serializer = getattr(settings, 'BGFILES_DEFAULT_SERIALIZER', Signer.DEFAULT_SERIALIZER)
        max_age = getattr(settings, 'BGFILES_DEFAULT_MAX_AGE', 86400)
        signer = Signer(key=key, salt=salt, serializer=serializer, max_age=max_age)
    return signer


def add_request(criteria='', file_type='', requester=None, content_type='', description=''):
    """Add a file request.

    :param criteria: the criteria that should be used to compose the file
    :type criteria: str
    :param file_type: the type of file to generate, e.g. "report-a" or "report-b"
    :type file_type: str
    :param requester: the instance of the user performing the requester
    :param content_type: the content or mime type of the resulting file
    :type content_type: str
    :param content_type: an optional description you can use to remind the user
    :type content_type: str
    :return: the file request
    """
    return FileRequest.objects.create(criteria=criteria, file_type=file_type, requester=requester,
                                      content_type=content_type, description=description, requested_at=timezone.now())


def attach_file(request, contents, filename=None, content_type=None, size=None):
    """Attach a file to a file request.

    This will mark the request as finished.

    :param request: the request
    :type request: bgfiles.models.FileRequest
    :param contents: the contents of the file/something that can be passed to a ``FileField``.
    :param filename: the filename; leave blank to maintain the current filename (if any)
    :type filename: str
    :param content_type: the content/mime type of the file; leave blank to maintain the current value (if any)
    :type content_type: str
    :param size: the size of the file in bytes; leave blank to maintain the current value (if any)
    :type size: int
    """
    request.attach_file(contents, filename, content_type, size=size)
    request.save()


def attach_local_file(request, filepath, filename=None, content_type=None, size=None):
    """Attach a local file to a file request.

    This will load the file and then pass it to ``attach_file``.

    :param request: the request
    :type request: bgfiles.models.FileRequest
    :param filepath: the full path to the file
    :type filepath: str
    :param filename: the filename; same as for ``attach_file``
    :type filename: str
    :param content_type: the content type; same as for ``attach_file``
    :type content_type: str
    :param size: the file size in bytes; same as for ``attach_file``
    :type size: int
    """
    with open(filepath, 'r') as contents:
        attach_file(request, File(contents), filename=filename, content_type=content_type, size=size)


def create_token(request, data=None, signer=None):
    """Create a requester-specific token for the request using the supplied signer.

    Creating a token this way requires the requester to be specified on the file request because their id
    will be included in the token. Otherwise a RuntimeError is raised.

    Note that the ``max_age`` property of the signer is used to set the expiration date of the file request.
    If you don't want to expire the request you should use a signer with no max age set.

    You can opt to add more data to the token and specify your own signer instance to use.

    :param request: file request
    :type request: bgfiles.models.FileRequest
    :param data: optional extra data to include in the token
    :type data: dict
    :param signer: signer to use; will default to the configured default signer
    :return: the token
    """
    if not request.requester_id:
        raise RuntimeError('Generating a requester token without a request is not possible')
    if not data:
        data = {}
    data['requester'] = request.requester_id
    return create_general_token(request, data, signer)


def create_general_token(request, data=None, signer=None):
    """Create a general token for the request using the supplied signer.

    In contrast with ``create_token`` this function does not require a requester to be set and won't embed
    their id even if a requester was specified in the request.

    Note that the ``max_age`` property of the signer is used to set the expiration date of the file request.
    If you don't want to expire the request you should use a signer with no max age set.

    You can opt to add more data to the token and specify your own signer instance to use.

    :param request: file request
    :type request: bgfiles.models.FileRequest
    :param data: optional extra data to include in the token
    :type data: dict
    :param signer: signer to use; will default to the configured default signer
    :return: the token
    """
    if not data:
        data = {}
    data['id'] = request.id
    signer = signer if signer else get_default_signer()
    token = signer.dump(data)
    if signer.max_age:
        request.expire_in(timedelta(seconds=signer.max_age))
        request.save()
    return token


def decode(token, require_requester=True, signer=None):
    """Decode a token.

    This turns the token back into the data you put in and returns a two-tuple containing the matching file
    request and that data. If the signer's ``max_age`` property is set this will also ensure the token isn't
    older than that max age (assuming you timestamp the token when you created it which is the default).

    If no matching file request is found, this will raise a ``FileRequest.DoesNotExist`` exception. When
    there's something wrong with the token the signer-specific exceptions will be raised. And if you require
    the requester id to be set but there's no such data in the token, this function will simply raise a
    ``BadSignature`` exception.

    :param token: token to decode
    :type token: str
    :param require_requester: whether we expect the requester id to be present in the data or not
    :type require_requester: bool
    :param signer: signer to use; defaults to the configured default signer
    :return: two-tuple (bgfiles.models.FileRequest, dict)
    """
    signer = signer if signer else get_default_signer()
    data = signer.load(token)
    filters = {'id': data['id']}
    if 'requester' in data:
        filters['requester_id'] = data['requester']
    elif require_requester:
        raise BadSignature()
    return FileRequest.objects.get(**filters), data


def serve(request, raise_on_missing=Http404, default_content_type='application/octet-stream'):
    """Serve the file attached to the request.

    Creates a simple HTTPResponse containing the attached file. When no file is attached, the ``raise_on_missing``
    exception will be raised, which defaults to ``Http404``. In case no content type was set on the file request,
    the ``default_content_type`` is used.

    :param request: the file request
    :type request: bgfiles.models.FileRequest
    :param raise_on_missing: exception to raise
    :type raise_on_missing: Exception
    :param default_content_type: content type to use when none was set on the file request
    :type default_content_type: str
    :return: an HTTP response
    """
    if not request.filehandle and raise_on_missing:
        raise raise_on_missing()
    content_type = request.content_type if request.content_type else default_content_type
    request.filehandle.open()
    try:
        response = HttpResponse(request.filehandle.file.read(), content_type=content_type)
    finally:
        request.filehandle.close()
    if request.filename:
        response['Content-Disposition'] = create_content_disposition(request.filename)
    return response



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function, division, absolute_import
from datetime import timedelta
from django.core.management import BaseCommand
from django.db import transaction
from django.utils import timezone
import time
from ...models import FileRequest
from ...patterns import registry


ACTION_PROCESS = 'process'
ACTION_CLEAN = 'clean'
DEFAULT_LEEWAY = 3600


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument('action', type='choices', choices=[ACTION_PROCESS, ACTION_CLEAN],
                            help='Either process outstanding requests or clean expired requests. Note that processing '
                                 'requires the usage of the registry')
        parser.add_argument('--sleep', type='float',
                            help='Optional (floating point) sleep in seconds between handling requests')
        parser.add_argument('--timeout', type='int',
                            help='Maximum time to run in seconds; this is only an approximation.')
        parser.add_argument('--items', type='int', help='Maximum number of items to process')
        parser.add_argument('--leeway', type='int', help='Number of seconds leeway to allow for expired files. This '
                                                         'should prevent interrupting last-minute file downloads',
                            default=DEFAULT_LEEWAY)

    def handle(self, *args, **options):
        action = options['action']
        sleep = options.get('sleep')
        max_items = options.get('items')
        timeout = options.get('timeout')
        timeout = timeout + time.time() if timeout else None
        if action == ACTION_PROCESS:
            self.process(sleep, timeout, max_items)
        elif action == ACTION_CLEAN:
            self.clean(options['leeway_expired'], sleep, timeout, max_items)
        else:
            raise RuntimeError('Unsupported action')

    def process(self, sleep, timeout, max_items):
        request_ids = FileRequest.objects.to_handle().values_list('id', flat=True)
        if max_items:
            request_ids = request_ids[:max_items]
        nr_items = 0
        for pk in request_ids:
            with transaction.atomic():
                generated = registry.dispatch(pk)
            if generated:
                nr_items += 1
                self.stdout.write('Handled request %s\n' % pk)
                if sleep:
                    time.sleep(sleep)
            else:
                self.stdout.write('Ignored request %s\n' % pk)
            if timeout and time.time() > timeout:
                self.stdout.write('Reached timeout. Stopping processing\n')
                break
        self.stdout.write('Handled %s requests' % nr_items)

    def clean(self, leeway, sleep, timeout, max_items):
        treshold = timezone.now()
        if leeway:
            treshold -= timedelta(seconds=leeway)
        with transaction.atomic():
            status, nr_deleted, total = FileRequest.objects.remove_expired(treshold, sleep, timeout, max_items)
        kwargs = {'status': status, 'nr_deleted': nr_deleted, 'total': total}
        if status == 'processed':
            self.stdout.write('Removed all {nr_deleted} expired files\n'.format(**kwargs))
        elif status == 'max_items':
            self.stdout.write('Stopped short because of max items: {nr_deleted}/{total}\n'.format(**kwargs))
        elif status == 'timeout':
            self.stdout.write('Stopped short because of timeout: {nr_deleted}/{total}\n'.format(**kwargs))
        else:
            self.stderr.write('Unknown exit status for cleaning: "{status}"\n'.format(**kwargs))



# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-01-20 17:57
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='FileRequest',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('criteria', models.TextField(blank=True)),
                ('filehandle', models.FileField(blank=True, null=True, upload_to='')),
                ('filename', models.CharField(blank=True, max_length=200)),
                ('description', models.TextField(blank=True)),
                ('file_type', models.CharField(blank=True, db_index=True, max_length=100)),
                ('content_type', models.CharField(blank=True, max_length=200)),
                ('size', models.BigIntegerField(blank=True, null=True)),
                ('requested_at', models.DateTimeField(db_index=True)),
                ('finished_at', models.DateTimeField(blank=True, db_index=True, null=True)),
                ('expires_at', models.DateTimeField(blank=True, null=True)),
                ('requester', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='requested_downloads', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['-requested_at'],
            },
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function, division, absolute_import
from django.utils.encoding import force_bytes, force_text
from django.utils.http import urlquote
import unicodedata


def create_content_disposition(filename, attachment=True):
    """Create the Content-Disposition HTTP header.

    This only supports ASCII and UTF-8 encoded strings. For UTF-8 encoded strings a fallback is added for
    older browsers.

    :param filename: filename to supply
    :type filename: str
    :param attachment: whether to force a dialog or not
    :type attachment: bool

    """
    # Start by replacing characters that might cause issues in some browsers
    cleaned = force_text(filename if filename else '')
    for ch in '/\\";':
        cleaned = cleaned.replace(ch, ' ')
    cleaned = force_bytes(cleaned)
    # Now try to guess the utf-8 and ascii variants of the filename
    utf8 = None
    ascii = attempt_decode(cleaned, 'ascii')
    # If ascii is set, we're dealing with a plain ascii string and no further processing is needed.
    if not ascii:
        utf8 = attempt_decode(cleaned, 'utf-8')
        if not utf8:
            # So the filename is utf-8 encoded; now try to get a sane ascii version for older browsers
            try:
                ascii = unicodedata.normalize('NFKD', utf8).encode('ascii', 'ignore')
            except UnicodeDecodeError:
                pass
    # The string's not ascii nor utf-8. We can only hope for the best.
    if not ascii:
        ascii = cleaned
    ascii = force_text(ascii)
    format_params = {
        'serve_type': 'attachment' if attachment else 'inline',
        'ascii': ascii
    }
    if utf8 and ascii:
        format_params['utf8'] = urlquote(utf8)
        header_format = '{serve_type}; filename="{ascii}"; filename*=UTF-8\'\'{utf8}'
    else:
        header_format = '{serve_type}; filename="{ascii}"'
    return header_format.format(**format_params).encode('ascii')


def attempt_decode(s, encoding):
    try:
        return s.decode(encoding)
    except UnicodeDecodeError:
        return None



# -*- coding: utf-8 -*-
"""
Patterns providing shortcuts to schedule file request processing with Celery.

"""
from __future__ import unicode_literals, print_function, division, absolute_import
from .base import FullPattern


class ScheduleWithCeleryMixin(object):
    """Pattern mixin to schedule requests using Celery tasks."""

    # The task function to use
    task = None

    def schedule(self, file_request, http_request, dataset):
        """Implementation of the pattern's ``schedule`` method.

        :param file_request: the file request to schedule
        :param http_request: http request in case you need it
        :param dataset: the dataset in case you need it
        :return: the task result
        """
        params = [file_request, http_request, dataset]
        task = self.get_task(*params)
        return task.apply_async(**self.get_celery_kwargs(*params))

    def get_task(self, file_request, http_request, dataset):
        """Get the actual task function to call.

        Defaults to using the class ``task`` attribute.

        :param file_request: file request we want to schedule
        :param http_request: http request in case you need it
        :param dataset: the dataset in case you need it
        :return: the task function to use for scheduling
        """
        return self.task

    def get_task_args(self, file_request, http_request, dataset):
        """Get the args for the task; defaults to the id of the file request.

        :param file_request: file request to schedule
        :param http_request: http request in case you need it
        :param dataset: the dataset in case you need it
        :return: the args for the task
        """
        return [file_request.id]

    def get_task_kwargs(self, file_request, http_request, dataset):
        """Get the kwargs for the task.

        :param file_request: file request to schedule
        :param http_request: http request in case you need it
        :param dataset: the dataset in case you need it
        :return: the kwargs (dict) for the task
        """
        return {}

    def get_celery_kwargs(self, file_request, http_request, dataset):
        """Get the Celery kwargs for the task, passed to ``apply_async``.

        :param file_request: file request to schedule
        :param http_request: http request in case you need it
        :param dataset: the dataset in case you need it
        :return: the Celery options (dict) for the task
        """
        return {
            'args': self.get_task_args(file_request, http_request, dataset),
            'kwargs': self.get_task_kwargs(file_request, http_request, dataset)
        }


class ScheduleWithCeleryPattern(ScheduleWithCeleryMixin, FullPattern):
    """Final class using the Celery mixin and the FullPattern."""

    def evaluate_dataset(self, dataset, http_request):
        raise NotImplementedError()

    def write_bytes(self, dataset, output):
        raise NotImplementedError()

    def get_items(self, criteria):
        raise NotImplementedError()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function, division, absolute_import

from .base import (SimplePattern, WriterPattern, DelayedTemplateResponseMixin, EmailNotificationMixin,  # noqa
                   FullPattern, InvalidCriteria, InvalidForm, Criteria, QueryDictCriteria,  # noqa
                   QueryDictCriteriaWithRequester, Dataset)  # noqa



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function, division, absolute_import
from bgfiles.http import create_content_disposition
from django.test import SimpleTestCase


class CreateContentDispositionTest(SimpleTestCase):

    def test(self):
        header = create_content_disposition('Fußball.pdf')
        self.assertEqual(b'attachment; filename="Fuball.pdf"; filename*=UTF-8\'\'Fu%C3%9Fball.pdf', header)
        header = create_content_disposition('Fußball.pdf', attachment=False)
        self.assertEqual(b'inline; filename="Fuball.pdf"; filename*=UTF-8\'\'Fu%C3%9Fball.pdf', header)
        header = create_content_disposition(b'Fussball.pdf')
        self.assertEqual(b'attachment; filename="Fussball.pdf"', header)
        header = create_content_disposition(b'Fussball.pdf', attachment=False)
        self.assertEqual(b'inline; filename="Fussball.pdf"', header)
        expected = (b'attachment; filename="Leery  Jenkins  My Man .pdf"; '
                    b'filename*=UTF-8\'\'L%C3%A9%C3%ABr%C5%93%C3%B8y%20%20Jenkins%20%20My%20Man%20.pdf')
        self.assertEqual(create_content_disposition('Léërœøy \\Jenkins/"My Man".pdf'), expected)
        expected = (b'inline; filename="Leery  Jenkins  My Man .pdf"; '
                    b'filename*=UTF-8\'\'L%C3%A9%C3%ABr%C5%93%C3%B8y%20%20Jenkins%20%20My%20Man%20.pdf')
        self.assertEqual(create_content_disposition('Léërœøy \\Jenkins/"My Man".pdf', attachment=False), expected)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from bgfiles.models import FileRequest, get_storage, get_upload_to
from datetime import timedelta

from django.core.files.base import ContentFile
from django.test import TestCase, SimpleTestCase, override_settings
from django.utils import timezone
import mock
import os
import time
from .utils import WithTempDir


class FileRequestSettingsTest(SimpleTestCase):

    def test_get_storage_default(self):
        self.assertTrue(get_storage() is None)

    def test_get_storage_overridden(self):
        sentinel = object()
        with override_settings(BG_FILES_STORAGE=sentinel):
            self.assertTrue(get_storage() is sentinel)

    def test_get_upload_to_default(self):
        self.assertEqual('', get_upload_to())

    def test_get_upload_to_overridden(self):
        upload_dir = '/oh/my/temp/'
        with override_settings(BG_FILES_UPLOAD_TO=upload_dir):
            self.assertEqual(upload_dir, get_upload_to())


class FileRequestManagerToHandleTest(TestCase):

    def test(self):
        now = timezone.now()
        # This one should be ignored
        FileRequest.objects.create(finished_at=now, requested_at=now)
        requests_to_handle = [
            FileRequest.objects.create(requested_at=now - timedelta(seconds=30)),
            FileRequest.objects.create(requested_at=now),
            FileRequest.objects.create(requested_at=now + timedelta(seconds=30)),
        ]
        results = list(FileRequest.objects.to_handle())
        self.assertEqual(3, len(results))
        self.assertEqual([req for req in requests_to_handle], results)


class FileRequestManagerRemoveExpiredTest(TestCase):

    def setUp(self):
        self.now = timezone.now()
        # This one should be ignored
        self.no_expiration = FileRequest.objects.create(expires_at=None, requested_at=self.now)
        self.expires_now = FileRequest.objects.create(expires_at=self.now, requested_at=self.now)
        self.treshold = self.now - timedelta(seconds=10)
        self.expired = [
            FileRequest.objects.create(expires_at=self.treshold - timedelta(seconds=300), requested_at=self.now),
            FileRequest.objects.create(expires_at=self.treshold - timedelta(seconds=30), requested_at=self.now),
            FileRequest.objects.create(expires_at=self.treshold, requested_at=self.now),
        ]

    def test(self):
        status, nr_deleted, total = FileRequest.objects.remove_expired(self.treshold)
        self.assertEqual(status, 'processed')
        self.assertEqual(nr_deleted, len(self.expired))
        self.assertEqual(nr_deleted, total)
        self.assertDeleted(self.expired)
        self.assertNotDeleted([self.expires_now, self.no_expiration])

    def test_max_items(self):
        status, nr_deleted, total = FileRequest.objects.remove_expired(self.treshold, max_items=1)
        self.assertEqual(status, 'max_items')
        self.assertEqual(nr_deleted, 1)
        self.assertEqual(total, len(self.expired))
        # The first request in expired should be the one that's deleted
        self.assertDeleted(self.expired[:1])
        self.assertNotDeleted(self.expired[1:] + [self.expires_now, self.no_expiration])

    def test_timeout(self):
        timeout = time.time()
        status, nr_deleted, total = FileRequest.objects.remove_expired(self.treshold, timeout=timeout)
        self.assertEqual(status, 'timeout')
        # Because we only check the timeout after the first run (what's the point otherwise?) we expect a single delete
        self.assertEqual(nr_deleted, 1)
        self.assertEqual(total, len(self.expired))
        # The first request in expired should be the one that's deleted
        self.assertDeleted(self.expired[:1])
        self.assertNotDeleted(self.expired[1:] + [self.expires_now, self.no_expiration])

    @mock.patch('bgfiles.models.time')
    def test_sleep(self, time_mock):
        sleep = 2.5
        status, nr_deleted, total = FileRequest.objects.remove_expired(self.treshold, sleep=sleep)
        self.assertEqual(status, 'processed')
        self.assertEqual(nr_deleted, len(self.expired))
        self.assertEqual(total, len(self.expired))
        self.assertDeleted(self.expired)
        self.assertNotDeleted([self.expires_now, self.no_expiration])
        # Verify sleep was called between requests
        time_mock.sleep.assert_called_with(sleep)
        self.assertEqual(len(self.expired) - 1, time_mock.sleep.call_count)

    def assertDeleted(self, requests):
        self.assertFalse(FileRequest.objects.filter(id__in=[r.id for r in requests]).exists())

    def assertNotDeleted(self, requests):
        self.assertEqual(len(requests), FileRequest.objects.filter(id__in=[r.id for r in requests]).count())


class FileRequestTest(WithTempDir, TestCase):

    def test_str(self):
        request = FileRequest(requested_at=timezone.now(), requester=None, file_type='report')
        expected = '{} requested at {} by {}'.format(request.file_type, request.requested_at, request.requester)
        self.assertEqual(expected, '%s' % request)

    def test_expire_in(self):
        request = FileRequest(requested_at=timezone.now(), requester=None, file_type='report')
        now = timezone.now()
        delta = timedelta(days=4, seconds=10)
        request.expire_in(delta, timestamp=now)
        self.assertEqual(request.expires_at, now + delta)
        request.expire_in(delta)
        self.assertTrue(request.expires_at >= now + delta)

    def test_attach_file_and_delete(self):
        request = FileRequest(requested_at=timezone.now(), requester=None, file_type='report')
        request.save()
        with override_settings(MEDIA_ROOT=self.tempdir, MEDIA_URL=''):
            contents = ContentFile('something special')
            request.attach_file(contents)
            request.save()
            self.assertTrue(request.finished_at)
            path = request.filehandle.path
            self.assertTrue(os.path.isfile(path))
            request.delete()
            self.assertFalse(os.path.isfile(path))

    def test_attach_file_with_overrides(self):
        values = {
            'filename': ('', 'abc', 'def', ''),
            'content_type': ('', '', 'xyz', 'oh'),
            'size': (None, 5, 0, 20)
        }
        request = FileRequest(requested_at=timezone.now(), requester=None, file_type='report')
        request.filename = 'ohboy'
        request.save()
        with override_settings(MEDIA_ROOT=self.tempdir, MEDIA_URL=''):
            contents = ContentFile('something special')
            for i in range(4):
                kwargs = {'contents': contents}
                old_values = {}
                new_values = {}
                for field, value_list in values.items():
                    old_values[field] = getattr(request, field)
                    new_values[field] = value_list[i]
                    kwargs[field] = value_list[i]
                request.attach_file(**kwargs)
                for field, old_value in old_values.items():
                    new_value = new_values[field]
                    if new_value:
                        self.assertEqual(new_value, getattr(request, field))
                    else:
                        self.assertEqual(old_value, getattr(request, field))
